import 'package:test/test.dart';
import 'package:intl/intl.dart';

import 'package:localizely_sdk/src/model/label.dart';

void main() {
  setUp(() {
    Intl.defaultLocale =
        'en'; // `getTranslation` method may use `Intl` lib for message processing (e.g. plural).
  });

  group('Label instantiation', () {
    test('Test instantiation with constructor when named args are not passed',
        () {
      var label = Label();

      expect(label.key, isNull);
      expect(label.value, isNull);
    });

    test('Test instantiation with constructor when named args are passed', () {
      var label = Label(key: 'key', value: 'value');

      expect(label.key, equals('key'));
      expect(label.value, equals('value'));
    });

    test('Test instantiation with named constructor when json arg is empty',
        () {
      var label = Label.fromJson({});

      expect(label.key, isNull);
      expect(label.value, isNull);
    });

    test(
        'Test instantiation with named constructor when json arg has partial data',
        () {
      var label = Label.fromJson({'key': 'key'});

      expect(label.key, equals('key'));
      expect(label.value, isNull);
    });

    test('Test instantiation with named constructor when json arg has all data',
        () {
      var label = Label.fromJson({'key': 'key', 'value': 'value'});

      expect(label.key, equals('key'));
      expect(label.value, equals('value'));
    });

    test(
        'Test instantiation with named constructor when json arg has additional data',
        () {
      var label = Label.fromJson(
          {'key': 'key', 'value': 'value', 'newData': 'new data'});

      expect(label.key, equals('key'));
      expect(label.value, equals('value'));
    });
  });

  group('Label literal translation', () {
    test('Test label literal translation when value is empty string', () {
      var label = Label(key: 'key', value: '');

      expect(label.getTranslation({}), isEmpty);
    });

    test('Test label literal translation when value is blank', () {
      var label = Label(key: 'labelKey', value: ' ');

      expect(label.getTranslation({}), equals(' '));
    });

    test('Test label literal translation when value is plain text', () {
      var label = Label(key: 'labelKey', value: 'plain text');

      expect(label.getTranslation({}), equals('plain text'));
    });

    test(
        'Test label literal translation when value is plain text which contains special chars',
        () {
      var label = Label(
          key: 'labelKey', value: 'chars: !@#\$%^&*()_+=-\\[];~`,.<>?:"\'');

      expect(label.getTranslation({}),
          equals('chars: !@#\$%^&*()_+=-\\[];~`,.<>?:"\''));
    });

    test(
        'Test label literal translation when value contains curly braces which are not indicating placeholder (unsupported)',
        () {
      var label = Label(key: 'key', value: '}{');

      expect(label.getTranslation({}), equals('}{'));
    }, skip: true);
  });

  group('Label argument translation', () {
    test('Test label argument translation when value contains just placeholder',
        () {
      var label = Label(key: 'key', value: '{placeholder}');

      expect(label.getTranslation({'placeholder': 'Some value'}),
          equals('Some value'));
    });

    test(
        'Test label argument translation when value contains plain text and placeholder',
        () {
      var label = Label(key: 'key', value: 'Hi {name}!');

      expect(label.getTranslation({'name': 'John'}), equals('Hi John!'));
    });

    test(
        'Test label argument translation when value contains plain text and few placeholders',
        () {
      var label = Label(
          key: 'key', value: 'My name is {lastName}, {firstName} {lastName}!');

      expect(label.getTranslation({'lastName': 'Bond', 'firstName': 'James'}),
          equals('My name is Bond, James Bond!'));
    });

    test(
        'Test label argument translation when value contains special chars and few placeholders',
        () {
      var label = Label(
          key: 'key',
          value:
              'chars: !@#\$%^&*()_+=-\\[];~`,.<>?:"\' {first} {second} {third}');

      expect(
          label.getTranslation({
            'third': 'third-val',
            'second': 'second-val',
            'first': 'first-val'
          }),
          equals(
              'chars: !@#\$%^&*()_+=-\\[];~`,.<>?:"\' first-val second-val third-val'));
    });

    test(
        'Test label argument translation when value contains placeholder and curly braces which are not indicating placeholder (unsupported)',
        () {
      var label = Label(key: 'key', value: '{placeholder} }{');

      expect(label.getTranslation({'placeholder': 'Some value'}),
          equals('Some value }{'));
    }, skip: true);
  });

  group('Label plural translation', () {
    test('Test label plural translation when value contains empty plural forms',
        () {
      var label = Label(
          key: 'labelKey',
          value:
              '{howMany, plural, zero {} one {} two {} few {} many {} other {}}');

      // Some plural forms are superfluous here (testing parsing)
      expect(label.getTranslation({'howMany': 1}), isEmpty);
      expect(label.getTranslation({'howMany': 5}), isEmpty);
    });

    test(
        'Test label plural translation when value contains plural forms with plain text',
        () {
      var label = Label(
          key: 'labelKey',
          value: '{howMany, plural, one {one-pf} other {other-pf}}');

      expect(label.getTranslation({'howMany': 1}), equals('one-pf'));
      expect(label.getTranslation({'howMany': 5}), equals('other-pf'));
    });

    test(
        'Test label plural translation when value contains plural forms with placeholder',
        () {
      var label = Label(
          key: 'labelKey',
          value:
              '{howMany, plural, one {one-pf: {placeholder}} other {other-pf: {placeholder}}}');

      expect(label.getTranslation({'howMany': 1, 'placeholder': 'Some value'}),
          equals('one-pf: Some value'));
      expect(label.getTranslation({'howMany': 2, 'placeholder': 'Some value'}),
          equals('other-pf: Some value'));
    });

    test(
        'Test label plural translation when value contains plural forms with few placeholders',
        () {
      var label = Label(
          key: 'labelKey',
          value:
              '{howMany, plural, one {one-pf: {howMany} {first} {second}} other {other-pf: {howMany} {first} {second}}}');

      expect(
          label.getTranslation(
              {'howMany': 1, 'first': 'first-val', 'second': 'second-val'}),
          equals('one-pf: 1 first-val second-val'));
      expect(
          label.getTranslation(
              {'howMany': 2, 'first': 'first-val', 'second': 'second-val'}),
          equals('other-pf: 2 first-val second-val'));
    });

    test(
        'Test label plural translation when value contains plural forms with special chars and few placeholders',
        () {
      var label = Label(
          key: 'labelKey',
          value:
              '{howMany, plural, one {one-pf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {howMany} {first}} other {other-pf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {howMany} {first}}}');

      // `<` char is not used in the test - cause parsing error
      expect(label.getTranslation({'howMany': 1, 'first': 'first-val'}),
          equals('one-pf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' 1 first-val'));
      expect(label.getTranslation({'howMany': 2, 'first': 'first-val'}),
          equals('other-pf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' 2 first-val'));
    });

    test(
        'Test label plural translation when value contains plural forms with lower sign (unsupported)',
        () {
      var label = Label(
          key: 'labelKey',
          value: '{howMany, plural, one {one-pf: <} other {other-pf: <}}');

      expect(label.getTranslation({'howMany': 1}), equals('one-pf: <'));
      expect(label.getTranslation({'howMany': 2}), equals('other-pf: <'));
    }, skip: true);
  });

  group('Label gender translation', () {
    test('Test label gender translation when value contains empty gender forms',
        () {
      var label = Label(
          key: 'key', value: '{gender, select, male {} female {} other {}}');

      expect(label.getTranslation({'gender': 'male'}), isEmpty);
      expect(label.getTranslation({'gender': 'female'}), isEmpty);
      expect(label.getTranslation({'gender': 'other'}), isEmpty);
    });

    test(
        'Test label gender translation when value contains gender forms with plain text',
        () {
      var label = Label(
          key: 'key',
          value:
              '{gender, select, male {male-gf} female {female-gf} other {other-gf}}');

      expect(label.getTranslation({'gender': 'male'}), equals('male-gf'));
      expect(label.getTranslation({'gender': 'female'}), equals('female-gf'));
      expect(label.getTranslation({'gender': 'other'}), equals('other-gf'));
    });

    test(
        'Test label gender translation when value contains gender forms with plain text and placeholder',
        () {
      var label = Label(
          key: 'key',
          value:
              '{gender, select, male {male-gf: {placeholder}} female {female-gf: {placeholder}} other {other-gf: {placeholder}}}');

      expect(
          label.getTranslation({'gender': 'male', 'placeholder': 'Some value'}),
          equals('male-gf: Some value'));
      expect(
          label.getTranslation(
              {'gender': 'female', 'placeholder': 'Some value'}),
          equals('female-gf: Some value'));
      expect(
          label
              .getTranslation({'gender': 'other', 'placeholder': 'Some value'}),
          equals('other-gf: Some value'));
    });

    test(
        'Test label gender translation when value contains gender forms with plain text and few placeholders',
        () {
      var label = Label(
          key: 'key',
          value:
              '{gender, select, male {male-gf: {gender} {first} {second}} female {female-gf: {gender} {first} {second}} other {other-gf: {gender} {first} {second}}}');

      expect(
          label.getTranslation(
              {'gender': 'male', 'first': 'first-val', 'second': 'second-val'}),
          equals('male-gf: male first-val second-val'));
      expect(
          label.getTranslation({
            'gender': 'female',
            'first': 'first-val',
            'second': 'second-val'
          }),
          equals('female-gf: female first-val second-val'));
      expect(
          label.getTranslation({
            'gender': 'other',
            'first': 'first-val',
            'second': 'second-val'
          }),
          equals('other-gf: other first-val second-val'));
    });

    test(
        'Test label gender translation when value contains gender forms with special chars and few placeholders',
        () {
      var label = Label(
          key: 'key',
          value:
              '{gender, select, male {male-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {gender} {first}} female {female-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {gender} {first}} other {other-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {gender} {first}}}');

      // `<` char is not used in the test - cause parsing error
      expect(
          label.getTranslation({'gender': 'male', 'first': 'first-val'}),
          equals(
              'male-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' male first-val'));
      expect(
          label.getTranslation({'gender': 'female', 'first': 'first-val'}),
          equals(
              'female-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' female first-val'));
      expect(
          label.getTranslation({'gender': 'other', 'first': 'first-val'}),
          equals(
              'other-gf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' other first-val'));
    });

    test(
        'Test label gender translation when value contains gender forms with lower sign (unsupported)',
        () {
      var label = Label(
          key: 'key',
          value:
              '{gender, select, male {male-gf: <} female {female-gf: <} other {other-gf: <}}');

      expect(label.getTranslation({'gender': 'male'}), equals('male-gf: <'));
      expect(
          label.getTranslation({'gender': 'female'}), equals('female-gf: <'));
      expect(label.getTranslation({'gender': 'other'}), equals('other-gf: <'));
    }, skip: true);
  });

  group('Label select translation', () {
    test('Test label select translation when value contains empty select forms',
        () {
      var label = Label(
          key: 'key', value: '{choice, select, foo {} bar {} baz {} other{}}');

      expect(label.getTranslation({'choice': 'foo'}), isEmpty);
      expect(label.getTranslation({'choice': 'bar'}), isEmpty);
      expect(label.getTranslation({'choice': 'baz'}), isEmpty);
      expect(label.getTranslation({'choice': 'ups'}), isEmpty);
    });

    test(
        'Test label select translation when value contains select forms with plain text',
        () {
      var label = Label(
          key: 'key',
          value:
              '{choice, select, foo {foo-sf} bar {bar-sf} baz {baz-sf} other{other-sf}}');

      expect(label.getTranslation({'choice': 'foo'}), equals('foo-sf'));
      expect(label.getTranslation({'choice': 'bar'}), equals('bar-sf'));
      expect(label.getTranslation({'choice': 'baz'}), equals('baz-sf'));
      expect(label.getTranslation({'choice': 'ups'}), equals('other-sf'));
    });

    test(
        'Test label select translation when value contains select forms with plain text and placeholder',
        () {
      var label = Label(
          key: 'key',
          value:
              '{choice, select, foo {foo-sf: {placeholder}} bar {bar-sf: {placeholder}} baz {baz-sf: {placeholder}} other{other-sf: {placeholder}}}');

      expect(
          label.getTranslation({'choice': 'foo', 'placeholder': 'Some value'}),
          equals('foo-sf: Some value'));
      expect(
          label.getTranslation({'choice': 'bar', 'placeholder': 'Some value'}),
          equals('bar-sf: Some value'));
      expect(
          label.getTranslation({'choice': 'baz', 'placeholder': 'Some value'}),
          equals('baz-sf: Some value'));
      expect(
          label.getTranslation({'choice': 'ups', 'placeholder': 'Some value'}),
          equals('other-sf: Some value'));
    });

    test(
        'Test label select translation when value contains select forms with plain text and few placeholders',
        () {
      var label = Label(
          key: 'key',
          value:
              '{choice, select, foo {foo-sf: {choice} {first} {second}} bar {bar-sf: {choice} {first} {second}} baz {baz-sf: {choice} {first} {second}} other{other-sf: {choice} {first} {second}}}');

      expect(
          label.getTranslation(
              {'choice': 'foo', 'first': 'first-val', 'second': 'second-val'}),
          equals('foo-sf: foo first-val second-val'));
      expect(
          label.getTranslation(
              {'choice': 'bar', 'first': 'first-val', 'second': 'second-val'}),
          equals('bar-sf: bar first-val second-val'));
      expect(
          label.getTranslation(
              {'choice': 'baz', 'first': 'first-val', 'second': 'second-val'}),
          equals('baz-sf: baz first-val second-val'));
      expect(
          label.getTranslation(
              {'choice': 'ups', 'first': 'first-val', 'second': 'second-val'}),
          equals('other-sf: ups first-val second-val'));
    });

    test(
        'Test label select translation when value contains select forms with special chars and few placeholders',
        () {
      var label = Label(
          key: 'key',
          value:
              '{choice, select, foo {foo-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {choice} {first}} bar {bar-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {choice} {first}} baz {baz-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {choice} {first}} other{other-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' {choice} {first}}}');

      // `<` char is not used in the test - cause parsing error
      expect(label.getTranslation({'choice': 'foo', 'first': 'first-val'}),
          equals('foo-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' foo first-val'));
      expect(label.getTranslation({'choice': 'bar', 'first': 'first-val'}),
          equals('bar-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' bar first-val'));
      expect(label.getTranslation({'choice': 'baz', 'first': 'first-val'}),
          equals('baz-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' baz first-val'));
      expect(
          label.getTranslation({'choice': 'ups', 'first': 'first-val'}),
          equals(
              'other-sf-chars: !@#\$%^&*()_+=-\\[];~`,.>?:"\' ups first-val'));
    });

    test(
        'Test label select translation when value contains select forms with lower sign (unsupported)',
        () {
      var label = Label(
          key: 'key',
          value:
              '{choice, select, foo {foo-sf: <} bar {bar-sf: <} baz {baz-sf: <} other{other-sf: <}}');

      expect(label.getTranslation({'choice': 'foo'}), equals('foo-sf: <'));
      expect(label.getTranslation({'choice': 'bar'}), equals('bar-sf: <'));
      expect(label.getTranslation({'choice': 'baz'}), equals('baz-sf: <'));
      expect(label.getTranslation({'choice': 'ups'}), equals('other-sf: <'));
    }, skip: true);
  });
}
